#ifndef	_GateNet_h_
#define	_GateNet_h_
#include <TNamed.h>
#include <TSocket.h>
#include <assert.h>

class GateNet : public TNamed {
private:
    TSocket * fSocket;
    union {
	Long64_t fBuffer64[2];
	UChar_t fBuffer8[16];
    };
public:
    GateNet() :
	fSocket(NULL) {
	memset(fBuffer64, 0xFF, sizeof(fBuffer64));
    };

    ~GateNet() {
	Close();
    };

    void Open(const char * addr, Int_t port = 23) {
	fSocket = new TSocket(addr, port);
	assert(fSocket->IsValid());
	fSocket->SetOption(kNoDelay, 1);
    };

    Bool_t IsOpened() const {
	if (fSocket)
	    return kTRUE;
	else
	    return kFALSE;
    };

    void Close() {
	if (fSocket)
	    fSocket->Close();
	delete fSocket;
	fSocket = NULL;
    };

    void RecvEvent();

    void SetData(UChar_t * buf, Int_t nbytes) {
	assert(nbytes <= sizeof(fBuffer8));
	memcpy(fBuffer8, buf, nbytes);
#if 0
	/* like ntohll */
	for (int i=0; i<nbytes; i++) {
	    int pos = 8*(i/8)+8-i%8-1;
	    fBuffer8[pos] = buf[i];
	}
#endif
    };

    Long64_t GetT0() {
	return fBuffer64[0];
    };

    Long64_t GetClock() {
	return fBuffer64[1];
    };

    void Print(Option_t * opt = "") {
	TNamed::Print();
#if 0
	printf("T0    = %08llx\n", GetT0());
	printf("Clock = %08llx\n", GetClock());
#endif
#if 1
	for (int i=0; i<16; i++) {
	    printf(" %02x", fBuffer8[i]);
	    if (i % 8 == 7)
		putchar('\n');
	}
#endif
    };

    TSocket * GetSocket() {
	return fSocket;
    };

    ClassDef(GateNet, 0)
};

#endif
