#ifndef	_HIT_H_
#define	_HIT_H_
#include <assert.h>
#include <TObject.h>
#include <Riostream.h>

class Hit : public TObject {
private:
    Int_t fCh;
    Long64_t fTime;
    Bool_t fHasADC;
    Int_t fADC;
public:
    Hit(Int_t ch = -1, Int_t time = -1, Int_t adc = -1) :
	fCh(ch), fTime(time) {
	if (adc == -1) {
	    fHasADC = kFALSE;
	    fADC = 0;
	} else {
	    fHasADC = kTRUE;
	    fADC = adc;
	}
    };

    Hit(Hit & hit) {
	if (&hit != this) {
	    fCh = hit.GetCh();
	    fTime = hit.GetTime();
	    fHasADC = hit.fHasADC;
	    fADC = hit.fADC;
	}
    };

    Bool_t HasADC() const {
	return fHasADC;
    };

    Int_t ADC() const {
	if (HasADC()) {
	    return fADC;
	} else {
	    return 0;
	}
    };

    virtual ~Hit() { };

    Int_t GetCh() const { return fCh; };
    Int_t GetTime() const { return fTime; };

    virtual void Print(Option_t * opt = "") const {
	cout << ClassName() << ":Ch=" << fCh << ' ' << "Time=" << fTime;
	cout << endl;
    };

    virtual Bool_t IsSortable() const { return kTRUE; };
    virtual Int_t Compare(const TObject * obj) const {
	if (obj->IsA() != Class())
	    return 1;

	Hit * p = (Hit *)obj;

	if (p->fTime < fTime) return 1;
	if (p->fTime > fTime) return -1;

	if (p->fCh < fCh) return 1;
	if (p->fCh > fCh) return -1;

	return 0;
    };

    ClassDef(Hit, 1);
};
#endif
