#ifndef _RECEIVER_H_
#define _RECEIVER_H_

#include <assert.h>

#include <TCondition.h>
#include <TRWLock.h>
#include <TSocket.h>
#include <TMessage.h>
#include <TObjArray.h>

#include <Riostream.h>

#include "BGClass.h"

class neighbor : public TObject {
public:
    TString host;
    Int_t port;
    Int_t wsize;
public:
    neighbor(char * h = "", Int_t p = -1, Int_t w = -1) : host(h), port(p), wsize(w) { };
    virtual ~neighbor() { };
    ClassDef(neighbor, 0)
};

class Receiver : public TObject {
protected:
    TObjArray fNeighbors;
    TObjArray fSockets;
    TObjArray fObjs;
    TCondition * fCondition;
public:

    Receiver() : fCondition(NULL) {
	fNeighbors.SetOwner(kTRUE);
	fSockets.SetOwner(kTRUE);
	fObjs.SetOwner(kTRUE);
    };

    virtual void Close() {
cerr << __PRETTY_FUNCTION__ << endl;
	TIter next(&fSockets);
	while (TObject * o = next()) {
	    assert(o->IsA() == TSocket::Class());
	    TSocket * s = (TSocket *)o;
	    cerr << "closing " << s->GetUrl() << endl;
	    s->Close();
	}
	fSockets.Clear();
    };

    virtual void Reset() {
cerr << __PRETTY_FUNCTION__ << endl;
	Close();
	fObjs.Clear();
    };

    virtual void Clear(Option_t * opt = "") {
cerr << __PRETTY_FUNCTION__ << endl;
	Reset();
	fNeighbors.Clear();
    };

    virtual ~Receiver() {
cerr << __PRETTY_FUNCTION__ << endl;
	Clear();
    };

    virtual void Add(char * name, Int_t port, Int_t wsize = -1) {
	fNeighbors.Add(new neighbor(name, port, wsize));
    };

    virtual Int_t Open() {
	Int_t ret = 0;
	TIter next(&fNeighbors);
	while (TObject * o = next()) {
	    neighbor * n = (neighbor *)o;
	    cerr << "connecting " << n->host << ':' << n->port << endl;
	    TSocket * s = new TSocket(n->host, n->port, n->wsize);
	    if (s->IsValid()) {
		fSockets.Add(s);
		ret++;
	    } else {
		cerr << "failed" << endl;
		delete s;
	    }
	}
	return ret;
    };

    virtual TObjArray & GetObjs() {
	return fObjs;
    };

    virtual void ReadEvent() {
	TIter iter(&fSockets);
	while (TObject * o = iter()) {
	    assert(o->IsA() == TSocket::Class());
	    TSocket * s = (TSocket *)o;
	    if (!s->IsValid())
		continue;

	    TMessage * msg = NULL;
	    s->Recv(msg);
	    assert(msg);
	    TClass * classinfo = msg->GetClass();
	    assert(classinfo);
	    TObject * oo = msg->ReadObject(classinfo);
	    fObjs.Add(oo);
	    delete msg;
	}
    };

    virtual Bool_t HasEvent() {
	TIter iter(&fSockets);
	while (TObject * o = iter()) {
	    assert(o->IsA() == TSocket::Class());
	    TSocket * s = (TSocket *)o;
	    assert(s->IsValid());
	    if (s->Select(TSocket::kRead, 5) == 0)
		return kFALSE;
	}
	return kTRUE;
    };

    virtual void Flush() {
	fObjs.Clear();
    };

    ClassDef(Receiver, 0)
};
#endif
