#include <TSystem.h>
#include <TGClient.h>
#include <assert.h>
// #include "BGClass.h"
#include "Run44Decoder.h"
#include <TCanvas.h>
#include <TRef.h>
#include <TSocket.h>
#include <TDirectory.h>
#include <TFile.h>

#include <TGFrame.h>
#include <TGButton.h>

#include <TGLabel.h>

#include <TInterpreter.h>

#include <TStopwatch.h>
#include <TMutex.h>

#include "where.h"


class mybutton : public TGTextButton {
private:
    void (*fFunc)(void *);
    void * fData;
public:
    mybutton(TGFrame * frame, char * text, Int_t id) :
	TGTextButton(frame, text, id),
	fFunc(NULL),
	fData(NULL)
    {
	SetFont("-adobe-courier-bold-r-*-*-34-*-*-*-*-*-iso8859-1");
    };

    void SetFunction (void (*func)(void *)) {
	fFunc = func;
    };

    void SetData(void * data) {
	fData = data;
    };

    virtual void Pressed() {
	TGTextButton::Pressed();
	cout << GetString() << " Pressed" << endl;
	if (fFunc)
	    fFunc(fData);
	cout << GetString() << " Done" << endl;
    };
};

class ResetPanel {
private:
    TGMainFrame fMain;
public:
    mybutton fReset;
#if 0
    mybutton fPause;
    mybutton fResume;
    mybutton fDump;
    mybutton fReload;
#endif

    ResetPanel() :
	fMain(gClient->GetRoot(), 200, 200),
	fReset(&fMain,  "Reset ", 1)
#if 0
	fPause(&fMain,  "Pause ", 2),
	fResume(&fMain, "Resume", 3),
	fDump(&fMain,   "Dump  ", 4),
	fReload(&fMain, "Reload", 5)
#endif
    {
      printf("Entering ResetPanel()\n");
	fMain.SetWindowName("Reset");
	fMain.AddFrame(&fReset);
#if 0
	fMain.AddFrame(&fPause);
	fMain.AddFrame(&fResume);
	fMain.AddFrame(&fDump);
	fMain.AddFrame(&fReload);
#endif

	fMain.SetWMPosition(386,260);

	fMain.MapSubwindows();
	fMain.Resize();
	fMain.MapWindow();

	// gSystem->ProcessEvents();
      printf("Exitting ReserPanel()\n");

    };

    ~ResetPanel() { };
};

class NhitPanel {
private:
    TGMainFrame fMain;
    TGLabel fRunNumber;
    TGLabel fStart;
    TGLabel fPresentTime;
    TGLabel fState;
    TGLabel fNevent;
    TGLabel fSingle_per_100;
    TGLabel fDouble_per_100;
    TGLabel fPositron_per_100;
    TGLabel fMuon_per_100;
    TGLabel fSingle;
    TGLabel fDouble;
    TGLabel fSingleFWD_per_100;
    TGLabel fSingleBWD_per_100;
    TGLabel fDoubleFWD_per_100;
    TGLabel fDoubleBWD_per_100;

public:
    NhitPanel() :
	fMain(gClient->GetRoot(), 200, 200),
	fRunNumber(&fMain,        "Run:                             "),
	fState(&fMain,            "RUN is stopped ---"),
	fStart(&fMain,            "Started at:                        "),
	fPresentTime(&fMain,      "Present at:                        "),
	fNevent(&fMain,           "# of Pulse:                      "),
	fSingle_per_100(&fMain,   "Single:            (hit/100pulse)"),
	fDouble_per_100(&fMain,   "Double:            (hit/100pulse)"),
	fPositron_per_100(&fMain, "Positron:          (hit/100pulse)"),
	fMuon_per_100(&fMain,     "Muon:              (hit/100pulse)"),
	fSingle(&fMain,           "Single:            (in this run)"),
	fDouble(&fMain,           "Double:            (in this run)"), 
	fSingleFWD_per_100(&fMain,"SingleFWD:         (hit/100pulse)"),
	fSingleBWD_per_100(&fMain,"SingleBWD:         (hit/100pulse)"),
	fDoubleFWD_per_100(&fMain,"DoubleFWD:         (hit/100pulse)"),
	fDoubleBWD_per_100(&fMain,"DoubleBWD:         (hit/100pulse)") {
	  printf("Entering NhitPanel()\n");

	fMain.SetWindowName("Nhit");

	const char font[] = "-adobe-courier-bold-r-*-*-34-*-*-*-*-*-iso8859-1";
	fRunNumber.SetTextFont(font);
	fState.SetTextFont(font);
	fStart.SetTextFont(font);
	fPresentTime.SetTextFont(font);
	fNevent.SetTextFont(font);
	fSingle_per_100.SetTextFont(font);
	fDouble_per_100.SetTextFont(font);
	fPositron_per_100.SetTextFont(font);
	fMuon_per_100.SetTextFont(font);
	fSingle.SetTextFont(font);
	fDouble.SetTextFont(font);
	fSingleFWD_per_100.SetTextFont(font);
	fSingleBWD_per_100.SetTextFont(font);
	fDoubleFWD_per_100.SetTextFont(font);
	fDoubleBWD_per_100.SetTextFont(font);

	fMain.SetWMPosition(0, 0);

	fMain.AddFrame(&fRunNumber);
	fMain.AddFrame(&fState);
	fMain.AddFrame(&fStart);
	fMain.AddFrame(&fPresentTime);
	fMain.AddFrame(&fNevent);
	fMain.AddFrame(&fSingle_per_100);
	fMain.AddFrame(&fDouble_per_100);
	fMain.AddFrame(&fPositron_per_100);
	fMain.AddFrame(&fMuon_per_100);
	fMain.AddFrame(&fSingle);
	fMain.AddFrame(&fDouble);
	fMain.AddFrame(&fSingleFWD_per_100);
	fMain.AddFrame(&fSingleBWD_per_100);
	fMain.AddFrame(&fDoubleFWD_per_100);
	fMain.AddFrame(&fDoubleBWD_per_100);
	fMain.MapSubwindows();
	fMain.Resize();
	fMain.MapWindow();
	  printf("Exitting NhitPanel()\n");

    };

    void SetRunNumber(Int_t run) {
	TString label;
	label.Form("Run:%8d", run);
	fRunNumber.SetText(label);
	fMain.Layout();
    };

    TString SetStart() {
	TDatime now;
	TString label;
	label.Form("Started at:%s", now.AsString());
	//	fStart.SetText(now.AsString());
	fStart.SetText(label);
	return label;
	fMain.Layout();
    };

    TString SetPresentTime() {
	TDatime now;
	TString label;
	label.Form("Present at:%s", now.AsString());
	//	fPresentTime.SetText(now.AsString());
	fPresentTime.SetText(label);
	return label;
	fMain.Layout();
    };

    void SetState(TString string) {
      fState.SetText(string);
      fMain.Layout();
    };

    void SetNevent(Int_t i) {
	TString label;
	label.Form("# of Pulse:%9d", i);
	fNevent.SetText(label);
	fMain.Layout();
    };

    void SetNeventWithBeam(Int_t i, Int_t j) {
	TString label;
	label.Form("# of Pulse:%9d%9d(on beam)", i, j);
	fNevent.SetText(label);
	fMain.Layout();
    };

    void SetSingle_per_100(Int_t i) {
	TString label;
	label.Form("Single: %12d(hit/100pulse)", i);
	fSingle_per_100.SetText(label);
	fMain.Layout();
    };

    void SetSingleFWD_per_100(Int_t i) {
	TString label;
	label.Form("SingleFWD: %9d(hit/100pulse)", i);
	fSingleFWD_per_100.SetText(label);
	fMain.Layout();
    };

    void SetSingleBWD_per_100(Int_t i) {
	TString label;
	label.Form("SingleBWD: %9d(hit/100pulse)", i);
	fSingleBWD_per_100.SetText(label);
	fMain.Layout();
    };

    void SetDouble_per_100(Int_t i) {
	TString label;
	label.Form("Double: %12d(hit/100pulse)", i);
	fDouble_per_100.SetText(label);
	fMain.Layout();
    };

    void SetDoubleFWD_per_100(Int_t i) {
	TString label;
	label.Form("DoubleFWD: %9d(hit/100pulse)", i);
	fDoubleFWD_per_100.SetText(label);
	fMain.Layout();
    };

    void SetDoubleBWD_per_100(Int_t i) {
	TString label;
	label.Form("DoubleBWD: %9d(hit/100pulse)", i);
	fDoubleBWD_per_100.SetText(label);
	fMain.Layout();
    };

    void SetPositron_per_100(Int_t i) {
	TString label;
	label.Form("Positron: %10d(hit/100pulse)", i);
	fPositron_per_100.SetText(label);
	fMain.Layout();
    };

    void SetMuon_per_100(Int_t i) {
	TString label;
	label.Form("Muon: %14d(hit/100pulse)", i);
	fMuon_per_100.SetText(label);
	fMain.Layout();
    };

    void SetSingle(Int_t i) {
	TString label;
	label.Form("Single: %12d(in this run)", i);
	fSingle.SetText(label);
	fMain.Layout();
    };

    void SetDouble(Int_t i) {
	TString label;
	label.Form("Double: %12d(in this run)", i);
	fDouble.SetText(label);
	fMain.Layout();
    };

    void Redraw() {
	if (gClient) {
	    fMain.Resize();
	    gClient->NeedRedraw(&fRunNumber);
	    //	    gClient->NeedRedraw(&fState);
	    gClient->NeedRedraw(&fStart);
	    gClient->NeedRedraw(&fPresentTime);
	    gClient->NeedRedraw(&fNevent);
	    gClient->NeedRedraw(&fSingle_per_100);
	    gClient->NeedRedraw(&fDouble_per_100);
	    gClient->NeedRedraw(&fPositron_per_100);
	    gClient->NeedRedraw(&fMuon_per_100);
	    gClient->NeedRedraw(&fSingle);
	    gClient->NeedRedraw(&fDouble);
	    gClient->NeedRedraw(&fSingleFWD_per_100);
	    gClient->NeedRedraw(&fSingleBWD_per_100);
	    gClient->NeedRedraw(&fDoubleFWD_per_100);
	    gClient->NeedRedraw(&fDoubleBWD_per_100);
	    gClient->NeedRedraw(&fMain);
	    // gSystem->ProcessEvents();
	}
    };
};

class Run44Display {
private:
    ResetPanel fResetPanel;
    NhitPanel  fNhitPanel;
    Run44Decoder fDecoder;
    TRef fCanvasRef;
    Bool_t	fIsRedrawing;
    Bool_t	fIsRunning;
    Bool_t	fIsPaused;
    TDirectory * fRootDirectory;
    TSocket * fNIMTDCConnection;
//    TMutex	fUpdateMutex;
public:
    Int_t	fRunNumber;
    TString    StartTime;
    Int_t      fTotalEventNumber; // is not cleared by Reset button
    Int_t	fEventNumber; // is cleard by Reset button
public:

    static void do_reset(void * arg) {
	if (arg) {
	    Run44Display * item = (Run44Display *)arg;
	    item->GetDecoder().Reset();
	    TString filename;

	    item->SetStart();

	    filename.Form("/home/daq/Data/reset.run.%d.event.%d.clock.%016llx",
		item->fRunNumber, item->fTotalEventNumber, item->GetDecoder().fClock);

	    item->fEventNumber = 0;

	    ofstream logfile(filename.Data());

	    logfile.setf( std::ios::hex, std::ios::basefield );
	    logfile.setf( std::ios::showbase );

	    logfile << item->GetDecoder().fT0 << endl;
	    logfile << item->GetDecoder().fClock << endl;
	}
    };

    void do_pause(void * arg) {
      if(N_COPPER_LITE_MODULES){
      TSocket s("192.168.0.1", 12000);//for VME controller 
	s.Close();
      }
      //      if(N_NIM_TDC_MODULES){
      for(int i=0; i<N_NIM_TDC_MODULES; i++){
	fNIMTDCConnection = new TSocket(nim_tdc_ip[i], "24");
      }
    };

    void do_resume(void * arg) {
      if(N_COPPER_LITE_MODULES){
	TSocket s("192.168.0.1", 12001);//for VME controller 
	s.Close();
      }
	if (fNIMTDCConnection) {
	    delete fNIMTDCConnection;
	    fNIMTDCConnection = NULL;
	}
    };

    void Pause() {
	do_pause(NULL);
	fIsPaused = kTRUE;
	UpdateNhit();
    };

    void Resume() {
	do_resume(NULL);
	fIsPaused = kFALSE;
	UpdateNhit();
    };


    static void do_dump(void * arg) {

	TString fname;

	TDatime date;
	fname.Form("/home/daq/Dump/dump_%d.%d.root",
		date.GetDate(), date.GetTime());

	TDirectory * pwd = gDirectory;
	TIter iter(pwd->GetList());

	TFile f(fname, "recreate");
	while (TObject * o = iter()) {
	    o->Write();
	}
	cout << "Dumped into " << fname << endl;
	pwd->cd();
    };

    static void do_reload(void * arg) {
	/* something to do */
      //      	gInterpreter->ExecuteMacro("/home/daq/monitorparams.cc");
      	gInterpreter->ExecuteMacro("/home/daq/monitorparams.cc");
    };

    void Dump(TMutex * mutex = NULL) {
	TDirectory * pwd = fRootDirectory;
	if (pwd == NULL)
	    return;
	assert(pwd);
	TDirectory * rawhistdir = pwd->GetDirectory("/");
	if (rawhistdir == NULL)
	    return;

	TIter iter(rawhistdir->GetList());

	const char filename0[] = "/dev/shm/monitor_status.new.root";
	const char filename1[] = "/dev/shm/monitor_status.root";
	//	const char filename0[] = "/dev/disk1/monitor_status.new.root";
	//	const char filename1[] = "/dev/disk1/monitor_status.root";

	unlink(filename0);


cerr << "lock1" << endl;
	if (mutex) mutex->Lock();
cerr << "lock2" << endl;
//TThread::Lock();
cerr << "create" << endl;
	TFile * monitor_file = new TFile(filename0, "recreate", "", 0);
cerr << "pwd" << endl;
	pwd->pwd();
cerr << "chdir" << endl;
	pwd->cd();
cerr << "unlock2" << endl;
//TThread::UnLock();
cerr << "unlock1" << endl;

	if (mutex) mutex->UnLock();
cerr << "done" << endl;
	TObject * o;

	while (o = iter()) {
	    if (o->InheritsFrom(TH1::Class())) {
		monitor_file->WriteObject(o, o->GetName());
	    }
	}

	if (mutex) mutex->Lock();
	monitor_file->Close();
	delete monitor_file;
	pwd->cd();
	if (mutex) mutex->UnLock();
	rename(filename0, filename1);

	ofstream neventfile("/home/daq/www/autorun/nevent.txt");
	neventfile << (fDecoder.fNevent) << endl;
	neventfile << (fDecoder.fNeventWithBeam);
    };

    virtual void Draw(Option_t * opt = "colz nostat") {
	TCanvas * c = (TCanvas *)fCanvasRef.GetObject();
	if (c) {
	    c->cd();
	    fDecoder.Draw(opt);
	}
    };

    Run44Display(Int_t nch = TOTAL_CHANNELS) :
	fDecoder(nch, 6400),
	fCanvasRef(new TCanvas("Monitor", "Monitor", -380, 190)),
	fRootDirectory(gDirectory),
	fIsRedrawing(kTRUE),
	fIsRunning(kFALSE),
	fIsPaused(kFALSE),
	fNIMTDCConnection(NULL)
    {
      printf("Entering Run44Display()\n");

	fResetPanel.fReset.SetFunction(&do_reset);
	printf("passed SetFunction\n");
	fResetPanel.fReset.SetData(this);
	printf("passed SetData\n");
#if 0
	fResetPanel.fPause.SetFunction(&do_pause);
	fResetPanel.fResume.SetFunction(&do_resume);
	fResetPanel.fDump.SetData(this);
	fResetPanel.fDump.SetFunction(&do_dump);
	fResetPanel.fReload.SetFunction(&do_reload);
#endif

	do_reload(NULL);
	printf("passed do_reload\n");
	gSystem->Sleep(500);

	((TCanvas *)(fCanvasRef.GetObject()))->SetWindowPosition(5, 830);

	// gSystem->ProcessEvents();
      printf("Exitting Run44Display()\n");

    };

    virtual ~Run44Display() {
	if (fCanvasRef.GetObject()) {
	    fCanvasRef.GetObject()->Delete();
	}
    };

    Run44Decoder & GetDecoder() {
	return fDecoder;
    };

    void UpdateCanvas() {

	printf("Entering UpdateCanvas()\n");

	// TLockGuard guard(&fUpdateMutex);

	fIsRedrawing = kTRUE;
	TCanvas * c = (TCanvas *)fCanvasRef.GetObject();
	if (c) {
	    c->Modified();
	    c->Update();
	} else {
	    printf("no canvas\n");
	}
	fIsRedrawing = kFALSE;

	printf("Exitting UpdateCanvas()\n");

    };

    void SetStart() {
	StartTime=fNhitPanel.SetStart();
    };

    void SetPresentTime() {
	fNhitPanel.SetPresentTime();
    };

    void SetRunning(Bool_t flag) {
	fIsRunning = flag;
    };

    Bool_t IsRunning() {
	return fIsRunning;
    };

    void UpdateNhit() {
        TString PresentTime;
	fNhitPanel.SetRunNumber(fRunNumber);

	printf("Entering UpdateNhits()\n");

	if (fIsRunning) {
	  if (fIsPaused) {
	    fNhitPanel.SetState("RUN is paused");
	  } else {
	    fNhitPanel.SetState("RUN is running");
	  }
	} else {
	  fNhitPanel.SetState("RUN is stopped");
	}

//	fNhitPanel.SetStart();
	PresentTime=fNhitPanel.SetPresentTime();
	//	fNhitPanel.SetPresentTime();
	//	fNhitPanel.SetNevent(fDecoder.fNevent);
	fNhitPanel.SetNeventWithBeam(fDecoder.fNevent,fDecoder.fNeventWithBeam);
	if(fIsRunning) {
	fNhitPanel.SetSingle_per_100(fDecoder.fNhit_single_per_100);
	fNhitPanel.SetDouble_per_100(fDecoder.fNhit_double_per_100);
	fNhitPanel.SetPositron_per_100(fDecoder.fNhit_positron_per_100);
	fNhitPanel.SetMuon_per_100(fDecoder.fNhit_muon_per_100);
	fNhitPanel.SetSingle(fDecoder.fNhit_single);
	fNhitPanel.SetDouble(fDecoder.fNhit_double);
	fNhitPanel.SetSingleFWD_per_100(fDecoder.fNhit_singleFWD_per_100);
	fNhitPanel.SetSingleBWD_per_100(fDecoder.fNhit_singleBWD_per_100);
	fNhitPanel.SetDoubleFWD_per_100(fDecoder.fNhit_doubleFWD_per_100);
	fNhitPanel.SetDoubleBWD_per_100(fDecoder.fNhit_doubleBWD_per_100);
	}
	fNhitPanel.Redraw();

	ofstream f("/home/daq/www/autorun/statistics.txt");
	char fname[80],rname[10];
	strcpy(fname,"/home/daq/www/autorun/RUNS/");
	sprintf(rname,"%06d",fRunNumber);strcat(fname,rname);
	strcat(fname,".stat");
	printf(rname);
	ofstream ff(fname);
	f << fRunNumber << endl; ff << fRunNumber << endl;
	//	f << "StartTime" << endl;
	f << StartTime << endl;	ff << StartTime << endl;
	//	label=fNhitPanel.fPresentTime.GetText();
	//       	f << "PresentTime" << endl;
       	f << PresentTime << endl;       	ff << PresentTime << endl;
	f << fDecoder.fNevent << endl;	ff << fDecoder.fNevent << endl;
	f << fDecoder.fNhit_single_per_100 << endl;	ff << fDecoder.fNhit_single_per_100 << endl;
	f << fDecoder.fNhit_double_per_100 << endl;	ff << fDecoder.fNhit_double_per_100 << endl;
	f << fDecoder.fNhit_positron_per_100 << endl;	ff << fDecoder.fNhit_positron_per_100 << endl;
	f << fDecoder.fNhit_muon_per_100 << endl;	ff << fDecoder.fNhit_muon_per_100 << endl;
	f << fDecoder.fNhit_single << endl;	ff << fDecoder.fNhit_single << endl;
	f << fDecoder.fNhit_double << endl;	ff << fDecoder.fNhit_double << endl;

	if (fIsRunning) {
	    if (fIsPaused) {
		f << "paused" << endl;
	    } else {
		f << "running" << endl;
	    }
	} else {
	    f << "stopped" << endl;
	}
	ff << "------" << endl;

	f << fDecoder.fNhit_singleFWD_per_100 << endl;	ff << fDecoder.fNhit_singleFWD_per_100 << endl;
	f << fDecoder.fNhit_singleBWD_per_100 << endl;	ff << fDecoder.fNhit_singleBWD_per_100 << endl;
	f << fDecoder.fNhit_doubleFWD_per_100 << endl;	ff << fDecoder.fNhit_doubleFWD_per_100 << endl;
	f << fDecoder.fNhit_doubleBWD_per_100 << endl;	ff << fDecoder.fNhit_doubleBWD_per_100 << endl;
	f << fDecoder.fNeventWithBeam << endl;	ff << fDecoder.fNeventWithBeam << endl;

	fDecoder.fNhit_single_per_100 = 0;
	fDecoder.fNhit_double_per_100 = 0;
	fDecoder.fNhit_positron_per_100 = 0;
	fDecoder.fNhit_muon_per_100 = 0;
	fDecoder.fNhit_singleFWD_per_100 = 0;
	fDecoder.fNhit_singleBWD_per_100 = 0;
	fDecoder.fNhit_doubleFWD_per_100 = 0;
	fDecoder.fNhit_doubleBWD_per_100 = 0;

	printf("Exitting UpdateNhits()\n");

    };

    void Update() {
      //	cout << "Update event=" << fDecoder.fNevent << endl;
	// UpdateNhit();
	UpdateCanvas();
	printf("passed UpdateCanvas()\n");
	fDecoder.UpdateAsymmetry();
	//	fDecoder.UpdateCut();
	printf("passed UpdateAsymmetry()\n");

    };
};
