#ifndef	_kalliope_tdc_h_
#define	_kalliope_tdc_h_

#include <Rtypes.h>

#include <assert.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>

#include <netdb.h>

#include <iostream>

#include "MLFdatum.h"

#define	KALLIOPE_TDC_BUFFER_SIZE (4*1024*1024)
#define	VBUFSIZE		(2 * EVENT_BUFFER_SIZE)

class tdc_hit {
public:
    /* only for little endian */
    unsigned short time;
    unsigned short info;
public:

    Int_t GetCh() const {
	Int_t rawch = info & 0x1F;
	return rawch;
    };

    Bool_t IsChLast() const {
	return (info & 0x20);
    };

    Bool_t IsOverflowed() const {
	return (info & 0x40);
    };

    UInt_t GetVal() const {
	return *(UInt_t *)(this);
    };

    Bool_t IsUp() {
	return true;
    };

    Bool_t IsDown() {
	return false;
    };
};

class kalliope_tdc_header {
public:
    UInt_t m_magic;
    UInt_t m_module_id; /* FINESSE slot in COPPER-Lite */
    UInt_t m_length;
    UInt_t m_event_id;

    UInt_t magic() const {
	return m_magic;
    };

    UInt_t module_id() const {
	return m_module_id;
    };

    UInt_t length() const {
	return m_length;
    };

    UInt_t event_id() const {
	return m_event_id;
    };

    void show(FILE * f = stdout) const {
	fprintf(f, "magic = %08x\n", magic());
	fprintf(f, "module_id = %08x\n", module_id());
	fprintf(f, "length = %08x\n", length());
	fprintf(f, "event_id = %08x\n", event_id());
    };
};

class kalliope_tdc_footer {
public:
    UInt_t m_value;

    kalliope_tdc_footer(int v = -1) {
	m_value = v;
    };

    UInt_t value() const {
	return m_value;
    };

    kalliope_tdc_footer operator=(UInt_t v) {
	m_value = v;
	return *this;
    };

};

class kalliope_tdc_event {
private:
    kalliope_tdc_header m_header;
    UChar_t *m_body;
    kalliope_tdc_footer m_footer;
    int m_len;

public:
    kalliope_tdc_event() {
	m_body = new UChar_t[KALLIOPE_TDC_BUFFER_SIZE];
    };

    kalliope_tdc_event(kalliope_tdc_header & header, int footer, UChar_t * p, int len) :
	m_header(header),
	m_footer(footer),
	m_len(len) {

	assert(m_len < KALLIOPE_TDC_BUFFER_SIZE);

	m_body = new UChar_t[KALLIOPE_TDC_BUFFER_SIZE];

	memcpy(m_body, p, len);
    };

    kalliope_tdc_event(const kalliope_tdc_event & e) {
	m_header = e.m_header;
	m_len = e.m_len;
	m_footer = e.m_footer;
	
	m_body = new UChar_t[KALLIOPE_TDC_BUFFER_SIZE];

	memcpy(m_body, e.m_body, m_len);
    };

    ~kalliope_tdc_event() {
	delete [] m_body;
    };

    void show() const {

	printf("=============kalliope_tdc_event.show()=======================\n");

	m_header.show();
	printf("footer = %08x\n", m_footer.value());

	assert(m_len % 4 == 0);

	int wlen = m_len / 4;

	UInt_t * p = (UInt_t *)m_body;

	for (int i=0; i<wlen; i++) {
	    if (i % 4 == 0)
		printf("%04x", i*4);
	    printf(" %08x", p[i]);
	    if (i % 4 == 3)
		fputc('\n', stdout);
	}

	if (wlen % 4 != 0)
	    fputc('\n', stdout);

	printf("====================================\n");
    };

    const kalliope_tdc_header & header() const {
	return m_header;
    };

    const kalliope_tdc_footer & footer() const {
	return m_footer;
    };

    const int len() const {
	return m_len;
    };

    UChar_t * body() const {
	return m_body;
    };

    void fill(kalliope_tdc_header & header, int footer, UChar_t * p, int len) {
	assert(len < KALLIOPE_TDC_BUFFER_SIZE);
	m_header = header;
	m_footer = footer;
	m_len = len;
	memcpy(m_body, p, len);
    };
};

class kalliope_tdc {
private:
    addrinfo * m_gai;
    int m_socket;
    FILE * m_fsocket;
    UChar_t * m_buffer;
    kalliope_tdc_event m_event;
    kalliope_tdc_header m_header;
    UInt_t m_footer;
    bool is_opened;
    Int_t m_module_id;
    char * m_vbuf;

public:

    kalliope_tdc() :
	m_gai(NULL),
	m_socket(-1),
	m_fsocket(NULL),
	is_opened(false) {
	m_buffer = new UChar_t[KALLIOPE_TDC_BUFFER_SIZE];
	m_vbuf = new char[VBUFSIZE];
    };

    kalliope_tdc(const char * host, const char * port) {
	m_buffer = new UChar_t[KALLIOPE_TDC_BUFFER_SIZE];
	m_vbuf = new char[VBUFSIZE];
	Open(host, port);
    };

    Int_t ModuleId() {
	return m_module_id;
    };

    void SetModuleId(Int_t i) {
	m_module_id = i;
    };

    int Open(const char * host, const char * port) {
	assert(is_opened == false);
	assert(m_gai == NULL);
	assert(m_fsocket == NULL);
	assert(m_socket == -1);

	addrinfo gai_hint;

	memset(&gai_hint, 0, sizeof(gai_hint));

	gai_hint.ai_protocol = PF_INET;
	gai_hint.ai_socktype = SOCK_STREAM;
	gai_hint.ai_protocol = IPPROTO_TCP;

	int gai_error = getaddrinfo(host, port, &gai_hint, &m_gai);

	if (gai_error) {
	    printf("%d %s\n", gai_error, gai_strerror(gai_error));
	    return (1);
	}

    m_socket = socket(m_gai->ai_family, m_gai->ai_socktype, m_gai->ai_protocol);
    
        int enable =1;
    
        int ret = setsockopt(m_socket, SOL_SOCKET, SO_KEEPALIVE, &enable, sizeof(enable));
    
        assert(ret == 0);
    
        ret = connect(m_socket, m_gai->ai_addr, m_gai->ai_addrlen);

	assert(ret == 0);

	m_fsocket = fdopen(m_socket, "r");

	ret = setvbuf(m_fsocket, m_vbuf, _IOFBF, VBUFSIZE);
	assert(ret == 0);

	is_opened = true;

	return 0;
    };

    void OpenFile(const char * filename) {
	m_fsocket = fopen(filename, "r");
	is_opened = true;
	m_socket = -1;
    };

    void Shutdown() {
	shutdown(m_socket, SHUT_RD);
    };

    void Close() {
	if (is_opened) {
	    freeaddrinfo(m_gai);
	    fclose(m_fsocket);
	    close(m_socket);
	    m_socket = -1;
	    m_gai = NULL;
	    m_fsocket = NULL;
	    is_opened = false;
	}
    };

    ~kalliope_tdc() {
	Close();
	delete [] m_buffer;
	delete [] m_vbuf;
    };

    void Read() throw(char const *) {

	if (!is_opened) {
	    throw("Instance of Kalliope TDC is not initialized");
	}

	int ret;

	ret = fread(&m_header, sizeof(m_header), 1, m_fsocket);
	if (ret != 1) {
	    fprintf(stderr, "reading header\n");
	    throw("reading header\n");
	}

	switch (m_header.magic()) {
	case 0xFFFF000A:
	case 0x00FF000A:
	case 0x01FF000A:
	    m_header.show(stderr);
	    fprintf(stderr, "bad copper header magic %08x\n", m_header.magic());
	case 0x7FFF000A:
	    break;
	default:
	    m_header.show(stderr);
	    fprintf(stderr, "bad copper header magic %08x\n", m_header.magic());
	    throw("bad header magic\n");
	}

	int len = m_header.length();

	if (len % 4 != 0 || len > 1024 * 1024 * 2) {
	    m_header.show();
	    fprintf(stderr, "bad length in header\n");
	    throw("bad length in header\n");
	}

	ret = fread(m_buffer, len, 1, m_fsocket);
	if (ret != 1) {
	    m_header.show();
	    fprintf(stderr, "reading body ret = %d ferror = %d\n", ret, ferror(m_fsocket));
	    throw("reading body\n");
	}

	ret = fread(&m_footer, sizeof(m_footer), 1, m_fsocket);
	if (ret != 1) {
	    fprintf(stderr, "reading m_footer\n");
	    throw("reading m_footer\n");
	}

	if (m_footer & 0x01000000) {
	    printf("****FIFO FULL CONDITION HAPPEN!!! (%08x)****\n", m_footer);
	    // has_error = true;
	}

	if ((m_footer >> 16) == 0xFF55) {
	    fprintf(stderr, "nword is short, I will read one more word\n");
	    UInt_t last_word;
	    ret = fread(&last_word, sizeof(last_word), 1, m_fsocket);
	    fprintf(stderr, "fread returned = %d, result %08x\n", ret, last_word);
	}

	m_event.fill(m_header, m_footer, m_buffer, len);
    };

    const kalliope_tdc_header & header() const {
	return m_header;
    };

    const UInt_t footer() const {
	return m_footer;
    };

    const kalliope_tdc_event & Last() {
	return m_event;
    };

#if 0
    void Decode(MLFdatum * destination) {
	int nhit = header().length();
	tdc_hit * hitlist = (tdc_hit *)(m_event.body());
	for (int i=0; i<nhit; i++) {
	    /* should be impletemented */
	}
    };
#endif

};
#endif
