#ifndef _GathererDecoder_h_
#define _GathererDecoder_h_

#include <TObjArray.h>

#include "FPGATDC32decoder.h"
// #include "rawevent.h"
#include "MLFdatum.h"

#include "copper_lite.h"
#include "kalliope_tdc.h"

class GathererDecoder : public FPGATDC32decoder {
private:
    Int_t fSlot;
    Int_t fN;
    Int_t fMaxhit;
    Int_t fOverFlow;
    MLFdatum * fBuffer; // [fMaxhit]
public:
    GathererDecoder(Int_t maxhit = 1024 * 256) :
	fSlot(0),
	fN(0),
	fMaxhit(maxhit),
	fBuffer(new MLFdatum[fMaxhit]) { };

    virtual ~GathererDecoder() {
	delete [] fBuffer;
    };

    virtual void OverFlowHandler(const FPGATDC32hit & raw) {
	fOverFlow++;
    };

    virtual void Push(Int_t ch, Int_t time, const FPGATDC32hit & raw) {
	fBuffer[fN++].Set(fSlot, ch, time);
    };

    virtual void Push(Long64_t val) {
	fBuffer[fN++].Set64(val);
    };

    void Reset() {
	fOverFlow = 0;
	fN = 0;
    };

    Int_t GetN() { return fN; };
    Int_t GetNbytes() { return fN * 8; };

    Int_t IsOverFlowed() { return fOverFlow; };

    void FillBuffer(UChar_t * buf, Int_t nbytes) {
	memcpy(buf, fBuffer, nbytes);
    };

    const UChar_t * const GetBuffer() const {
	return (const UChar_t * const)fBuffer;
    };

    void decode_copper_lite_event(const copper_lite_event & e, Int_t copper_slot) {
	fSlot = copper_slot;
	fBuffer[fN++].Set32(htonl(e.header().id())); /* event counter */
	fBuffer[fN++].Set32(htonl(e.header().finesse())); /* finesse slot */
	fBuffer[fN++].Set32(htonl(e.header().length())); /* nbytes */
	FPGATDC32decoder::decode_finesse((UInt_t *)e.body(), e.len()/4, e.header().finesse()>>24);
    };

    void decode_kalliope_tdc_event(const kalliope_tdc_event & e, Int_t module_id) {
	fSlot = module_id;
	fBuffer[fN++].Set32(htonl(e.header().event_id()));
	fBuffer[fN++].Set32(htonl(e.header().module_id()));
	fBuffer[fN++].Set32(htonl(e.header().length()));
	FPGATDC32decoder::decode_finesse((UInt_t *)e.body(), e.len()/4, e.header().module_id());
    };

};
#endif
