#include <TApplication.h>
#include <TFile.h>
#include <TObjArray.h>
#include <TBrowser.h>
#include <TKey.h>
#include <TH1.h>
#include <TH2.h>
#include <TROOT.h>
#include <TCanvas.h>
#include <Riostream.h>
#include "BGClass.h"

#include <assert.h>

class spy : public BGClass {
    TDirectory * fDir;
    TObjArray fList;
public:
    spy() : fDir(gDirectory) {
    };

    void h1copy(TH1 * hsrc) {
	assert(hsrc);
	TObject * dst = fDir->FindObject(hsrc->GetName());
	if (dst == NULL) {
	    TObject * o = hsrc->Clone();
	    /* hsrc is TH1, automatically added to the current Directory */
	    return;
	}

	assert(dst->InheritsFrom(TH1::Class()));

	TH1 * hdst = (TH1 *)dst;

	TThread::Lock();

	hdst->Reset("ICE");
	hdst->SetEntries(0);
	const Int_t nbinsX = hsrc->GetNbinsX();
	const Double_t xmin = hsrc->GetXaxis()->GetXmin();
	const Double_t xmax = hsrc->GetXaxis()->GetXmax();
	const Double_t xunit = (xmax - xmin)/nbinsX;

	for (int xbin=0; xbin<nbinsX+2; xbin++) {
	    hdst->Fill(xmin+xunit*(xbin-1), hsrc->GetBinContent(xbin));
	}

	hdst->SetEntries(hsrc->GetEntries());

	TThread::UnLock();
    };

    void h2copy(TH2 * hsrc) {
	assert(hsrc);
	TObject * dst = fDir->FindObject(hsrc->GetName());
	if (dst == NULL) {
	    TObject * o = hsrc->Clone();
	    /* hsrc is TH1, automatically added to the current Directory */
	    return;
	}

	assert(dst->InheritsFrom(TH2::Class()));

	TH2 * hdst = (TH2 *)dst;

	const int nbins = hsrc->GetMaximumBin();

	TThread::Lock();
	hdst->Reset("ICE");
	const Int_t nbinsX = hsrc->GetNbinsX();
	const Int_t nbinsY = hsrc->GetNbinsY();
	const Double_t xmin = hsrc->GetXaxis()->GetXmin();
	const Double_t xmax = hsrc->GetXaxis()->GetXmax();
	const Double_t ymin = hsrc->GetYaxis()->GetXmin();
	const Double_t ymax = hsrc->GetYaxis()->GetXmax();
	const Double_t xunit = (xmax - xmin)/nbinsX;
	const Double_t yunit = (ymax - ymin)/nbinsY;

	for (int ybin=0; ybin<nbinsY+2; ybin++) {
	    for (int xbin=0; xbin<nbinsX+2; xbin++) {
		hdst->Fill(xmin+xunit*(xbin-1), ymin+yunit*(ybin-1), hsrc->GetBinContent(xbin,ybin));
	    }
	}
	hdst->SetEntries(hsrc->GetEntries());
	TThread::UnLock();
    };

    virtual void ThreadFunc() {
	while (1) {
	    TFile f("/dev/shm/monitor_status.root");
	    fDir->cd();

	    TList * keylist = f.GetListOfKeys();
	    TIter key_iter(keylist);
	    while (TObject * k_o = key_iter()) {
		assert(k_o->IsA() == TKey::Class());
		TKey * key = (TKey *)k_o;

		TObject * o = f.Get(key->GetName());
		if (o->InheritsFrom(TH2::Class())) {
		    cout << o->GetName() << ':' << ((TH1 *) o)->GetEntries() << endl;
		    h2copy((TH2 *) o);
		} else if (o->InheritsFrom(TH1::Class())) {
		    // o->Print();
		    cout << o->GetName() << ':' << ((TH1 *) o)->GetEntries() << endl;
		    h1copy((TH1 *) o);
		}
	    }

	    f.Close();

#if 0
	    cout << "=====" << endl;
	    fDir->ls();
	    cout << "=====" << endl;
#endif

	    TIter iter_c(gROOT->GetListOfCanvases());
	    while (TCanvas * c = (TCanvas *)iter_c()) {
		TIter iter_p(c->GetListOfPrimitives());
		while (TObject * o = iter_p()) {
		    if (o->IsA() != TPad::Class())
			continue;
		    TPad * pad = (TPad *)o;
		    pad->Modified();
		}
		c->Modified();
		c->Update();
	    }

	    TThread::Sleep(5);
	}
    };
};

int main(int argc, char ** argv)
{
    TApplication app(argv[0], NULL, NULL);

    TBrowser b;

    gDirectory->cd("/");

    spy item;

    item.Start();

    app.Run();
}
